from imdby.utils import *


# Retrieves IMDb Critic Reviews
class critic_reviews:

    """
    Collects critic reviews of the multi media content in IMDb when titleid is given.
    :param user reviews: Unique identification titleid for every multi media content in IMDb.
    :returns: Returns all the user reviews.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.user_reviews_url = "https://www.imdb.com/title/%s/externalreviews" % self.titleid

        # Creating soup for the website
        soup = BeautifulSoup(get(self.user_reviews_url).text, 'lxml')

        """
        :returns: movie title if available.
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        :returns: total critic reviews if available.
        """
        try:
            self.critic_reviews_count = int(''.join(i for i in soup.select_one('.desc').text if i.isdigit()))
        except:
            self.critic_reviews_count = None

        """
        :returns: critics reviews link if available.
        """
        try:
            links = soup.select_one("#external_reviews_content").findNext("ul").select("li")
            self.critic_reviews_df = pd.DataFrame(columns=['Title', 'URL'])

            for item in links:
                try:
                    self.critic_reviews_df.loc[len(self.critic_reviews_df)] = [item.a.text.strip(),
                                                                               get('https://www.imdb.com%s' % item.a['href']).url]
                except:
                    pass
        except:
            self.critic_reviews_df = None

        print("\rCritic Reviews Extraction Completed\r", end="\r", flush=True)

# main class which passes the titleid to indiviual class
class imdb:
    def __init__(self, titleid):
        start_time = datetime.now()
        critic_reviews.__init__(self, titleid)

        time_delta = datetime.now() - start_time
        print("\rCalculating time taken for critic reviews extraction : %s  seconds\r" % (time_delta.seconds), end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb Top India Charts Details
class top_india_charts:

    """
    Collects top india charts details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self):
        base_url = "https://www.imdb.com/india/"

        self.top_rated_indian_movies_url =  "%stop-rated-indian-movies" % base_url
        self.top_rated_tamil_movies_url = "%stop-rated-tamil-movies" % base_url
        self.top_rated_telugu_movies_url = "%stop-rated-telugu-movies" % base_url
        self.top_rated_malayalam_movies_url = "%stop-rated-malayalam-movies" % base_url

        indian_soup = BeautifulSoup(get(self.top_rated_indian_movies_url).text, 'lxml')
        tamil_soup = BeautifulSoup(get(self.top_rated_tamil_movies_url).text, 'lxml')
        telugu_soup = BeautifulSoup(get(self.top_rated_telugu_movies_url).text, 'lxml')
        malayalam_soup = BeautifulSoup(get(self.top_rated_malayalam_movies_url).text, 'lxml')

        """
        :returns: top_rated_indian_movies DataFrame
        """
        try:
            top_250 = indian_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, directors, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(' '.join(title.contents[0].split()).replace('.',''))
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    directors.append([item.replace(' (dir.)', '').strip() for item in title.a['title'].split(',') if ' (dir.)' in item])
                except:
                    directors.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.top_rated_indian_movies_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids,
                                                            'Director' : directors, 'Main Cast' : main_cast,
                                                            'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                            'Poster' : posters})
        except:
            self.top_rated_indian_movies_df = None


        """
        :returns: top_rated_tamil_movies DataFrame
        """
        try:
            top_250 = tamil_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, directors, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(' '.join(title.contents[0].split()).replace('.',''))
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    directors.append([item.replace(' (dir.)', '').strip() for item in title.a['title'].split(',') if ' (dir.)' in item])
                except:
                    directors.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.top_rated_tamil_movies_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids,
                                                           'Director' : directors, 'Main Cast' : main_cast,
                                                           'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                           'Poster' : posters})
        except:
            self.top_rated_tamil_movies_df = None

        """
        :returns: top_rated_telugu_movies DataFrame
        """
        try:
            top_250 = telugu_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, directors, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(' '.join(title.contents[0].split()).replace('.',''))
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    directors.append([item.replace(' (dir.)', '').strip() for item in title.a['title'].split(',') if ' (dir.)' in item])
                except:
                    directors.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.top_rated_telugu_movies_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids,
                                                            'Director' : directors, 'Main Cast' : main_cast,
                                                            'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                            'Poster' : posters})
        except:
            self.top_rated_telugu_movies_df = None

        """
        :returns: top_rated_malayalam_movies DataFrame
        """
        try:
            top_250 = malayalam_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, directors, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(' '.join(title.contents[0].split()).replace('.',''))
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    directors.append([item.replace(' (dir.)', '').strip() for item in title.a['title'].split(',') if ' (dir.)' in item])
                except:
                    directors.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.top_rated_malayalam_movies_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids,
                                                               'Director' : directors, 'Main Cast' : main_cast,
                                                               'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                               'Poster' : posters})
        except:
            self.top_rated_malayalam_movies_df = None

        print("\rTop India Charts Extraction Completed\r", end="\r", flush=True)

# main class which passes the titleid to indiviual class
class imdb:
    def __init__(self):
        start_time = datetime.now()
        top_india_charts.__init__(self)

        time_delta = datetime.now() - start_time
        print("\rCalculating time taken for top india charts extraction : %s  seconds\r" % (time_delta.seconds), end="\r", flush=True)



from __future__ import print_function  # Only python 2.X

import re
import sys
import time
from datetime import datetime

import chromedriver_binary  # Adds chromedriver binary to path
from bs4 import BeautifulSoup
from requests import get
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.support.ui import WebDriverWait

import pandas as pd
from textblob import TextBlob
from vaderSentiment.vaderSentiment import SentimentIntensityAnalyzer



from imdby.utils import *


# Retrieves IMDb Charts Details
class imdb_charts:

    """
    Collects top rated movies details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self):
        base_url = "https://www.imdb.com/chart/"

        self.top_rated_movies_url = base_url + "top"
        self.top_rated_english_movies_url = base_url + "top-english-movies"
        self.top_box_office_us_url = base_url + "boxoffice"
        self.most_popular_movies_url = base_url + "moviemeter"
        self.lowest_rated_movies_url = base_url + "bottom"
        self.top_rated_tv_shows_url = base_url + "toptv"
        self.most_popular_tv_shows_url = base_url + "tvmeter"

        top_soup = BeautifulSoup(get(self.top_rated_movies_url).text, 'lxml')
        box_office_soup = BeautifulSoup(get(self.top_box_office_us_url).text, 'lxml')
        most_popular_movies_soup = BeautifulSoup(get(self.most_popular_movies_url).text, 'lxml')
        bottom_soup = BeautifulSoup(get(self.lowest_rated_movies_url).text, 'lxml')
        toptv_soup = BeautifulSoup(get(self.top_rated_tv_shows_url).text, 'lxml')
        most_popular_tv_shows_soup = BeautifulSoup(get(self.most_popular_tv_shows_url).text, 'lxml')
        top_english_soup = BeautifulSoup(get(self.top_rated_english_movies_url).text, 'lxml')

        """
        :returns: top_rated_movies DataFrame
        """
        try:
            top_250 = top_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, directors, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(' '.join(title.contents[0].split()).replace('.',''))
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    directors.append([item.replace(' (dir.)', '').strip() for item in title.a['title'].split(',') if ' (dir.)' in item])
                except:
                    directors.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.top_rated_movies_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids,
                                                     'Director' : directors, 'Main Cast' : main_cast,
                                                     'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                     'Poster' : posters})
        except:
            self.top_rated_movies_df = None

        """
        :returns: top_box_office_us_movies DataFrame
        """
        try:
            top_250 = box_office_soup.select_one('tbody').select('tr')
            names, ids, directors, main_cast, posters, weekend, gross, week = [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select('.ratingColumn')
                weeks = top_250[i].select_one('.weeksColumn')

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    directors.append([item.replace(' (dir.)', '').strip() for item in title.a['title'].split(',') if ' (dir.)' in item])
                except:
                    directors.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    weekend.append(' '.join(rating[0].text.split()))
                except:
                    weekend.append(None)

                try:
                    gross.append(' '.join(rating[1].text.split()))
                except:
                    gross.append(None)

                try:
                    week.append(weeks.text)
                except:
                    week.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    poster.append(None)

            self.top_box_office_df = pd.DataFrame({'Name' : names, 'ID' : ids,
                                                   'Director' : directors, 'Main Cast' : main_cast,
                                                   'Weekend' : weekend, 'Gross' : gross, 'Total Weeks' : week,
                                                   'Poster' : posters})
        except:
            self.top_box_office_df = None

        """
        :returns: top_box_office_us_movies weekend
        """
        try:
            self.top_box_office_weekend = box_office_soup.select_one('#boxoffice').h4.text
        except:
            self.top_box_office_weekend = None

        """
        :returns: most_popular_movies DataFrame
        """
        try:
            top_250 = most_popular_movies_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, directors, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(title.select_one('.velocity').text.split()[0])
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    directors.append([item.replace(' (dir.)', '').strip() for item in title.a['title'].split(',') if ' (dir.)' in item])
                except:
                    directors.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.most_popular_movies_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids,
                                                        'Director' : directors, 'Main Cast' : main_cast,
                                                        'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                        'Poster' : posters})
        except:
            self.most_popular_movies_df = None

        """
        :returns: lowest_rated_movies DataFrame
        """
        try:
            top_250 = bottom_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, directors, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(' '.join(title.contents[0].split()).replace('.',''))
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    directors.append([item.replace(' (dir.)', '').strip() for item in title.a['title'].split(',') if ' (dir.)' in item])
                except:
                    directors.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.lowest_rated_movies_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids,
                                                        'Director' : directors, 'Main Cast' : main_cast,
                                                        'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                        'Poster' : posters})
        except:
            self.lowest_rated_movies_df = None

        """
        :returns: top_rated_tv_shows DataFrame
        """
        try:
            top_250 = toptv_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(' '.join(title.contents[0].split()).replace('.',''))
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.top_rated_tv_shows_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids, 'Main Cast' : main_cast,
                                                       'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                       'Poster' : posters})
        except:
            self.top_rated_tv_shows_df = None

        """
        :returns: most_popular_tv_shows DataFrame
        """
        try:
            top_250 = most_popular_tv_shows_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(title.select_one('.velocity').text.split()[0])
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.most_popular_tv_shows_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids, 'Main Cast' : main_cast,
                                                          'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                          'Poster' : posters})
        except:
            self.most_popular_tv_shows_df = None

        """
        :returns: top_rated_english_movies DataFrame
        """
        try:
            top_250 = top_english_soup.select_one('.lister-list').select('tr')
            ranks, names, ids, directors, main_cast, years, ratings, rating_stats, posters = [], [], [], [], [], [], [], [], []

            for i in range(len(top_250)):
                title = top_250[i].select_one('.titleColumn')
                poster = top_250[i].select_one('.posterColumn')
                rating = top_250[i].select_one('.ratingColumn')

                try:
                    ranks.append(' '.join(title.contents[0].split()).replace('.',''))
                except:
                    ranks.append(None)

                try:
                    names.append(title.a.text)
                except:
                    names.append(None)

                try:
                    ids.append(poster.a['href'][7:16])
                except:
                    ids.append(None)

                try:
                    directors.append([item.replace(' (dir.)', '').strip() for item in title.a['title'].split(',') if ' (dir.)' in item])
                except:
                    directors.append(None)

                try:
                    main_cast.append([item.strip() for item in title.a['title'].split(',') if ' (dir.)' not in item])
                except:
                    main_cast.append(None)

                try:
                    years.append(re.findall("\d+", title.select_one('.secondaryInfo').text)[0])
                except:
                    years.append(None)

                try:
                    ratings.append(rating.strong.text)
                except:
                    ratings.append(None)

                try:
                    rating_stats.append(rating.strong['title'])
                except:
                    rating_stats.append(None)

                try:
                    posters.append(poster.a.img['src'])
                except:
                    posters.append(None)

            self.top_rated_english_movies_df = pd.DataFrame({'Rank' : ranks, 'Name' : names, 'ID' : ids,
                                                             'Director' : directors, 'Main Cast' : main_cast,
                                                             'Year' : years, 'Rating' : ratings, 'Rating_Stats' : rating_stats,
                                                             'Poster' : posters})
        except:
            self.top_rated_english_movies_df = None

        print("\rIMDb Charts Extraction Completed\r", end="\r", flush=True)

# main class which passes the titleid to indiviual class
class imdb:
    def __init__(self):
        start_time = datetime.now()
        imdb_charts.__init__(self)

        time_delta = datetime.now() - start_time
        print("\r Calculating time taken for IMDb Charts extraction : %s  seconds\r" % (time_delta.seconds), end="\r", flush=True)



from imdby.utils import *


class upcoming_releases:
    """
    'Upcoming Releases' method shows the available region in IMDb and suggests the regions to the user.
    Based upon user's selection it returns a upcoming_releases DataFrame.

    Input serial numbers separated with spaces.
    -------------------- -------------------- -------------------- -------------------- --------------------
    :returns: Upcoming Releases in selected regions.
    :raises ValueError: Raises an exception when entered a charater other than a number for selection.
    :raises IndexError: Raises an BeautifulSoup exception when index is out of range.
    """

    def __init__(self):

        """
        returns: Country Name & Country Code
        """
        url = 'https://www.imdb.com/calendar/'
        soup = BeautifulSoup(get(url).text, 'lxml')

        countries, country_code,  = [], []
        country = soup.select_one('#sidebar').select('a')

        try:
            for item in country:
                print('%s' %(country.index(item) + 1) + ': ' + item.text.strip())
                countries.append(item.text)
                country_code.append(item['href'][17:19].lower())

            input_name = re.findall(r"[\w']+", input('Enter serial number\t'))
            countries = [countries[int(load) - 1] if int(load) != 0 else '' for load in input_name]
            country_code = [country_code[int(load) - 1] if int(load) != 0 else '' for load in input_name]

            if len(country_code) == 1:
                self.country_name = countries[0]
                self.country_code = country_code[0]
            else:
                self.country_name = countries
                self.country_code = country_code

        except Exception as es:
            print("{0} :".format(type(es)), es)
            sys.exit(0)

        """
        returns: Upcoming Release for selected regions
        """
        self.region_url = 'https://www.imdb.com/calendar/?region=%s' % self.country_code
        region_soup = BeautifulSoup(get(self.region_url).text, 'lxml')

        release_dates = region_soup.select_one('#pagecontent').select('h4')
        movie_name, title_id, release_date, year = [None] * 4
        self.upcoming_releases_df = pd.DataFrame(columns=['Release Date', 'Movie Title' , 'ID' , 'Years' ])

        for item in release_dates:

            try:
                movies = item.findNext('ul').select('a')
                years = item.findNext('ul').select('li')
                
                for i in zip(movies, years):
                    release_date = item.text
                    movie_name = i[0].text.strip()
                    title_id = i[0]['href'][7:16]
                    year = re.findall("\d+", i[1].contents[2].strip())[0]
                    self.upcoming_releases_df.loc[len(self.upcoming_releases_df)] = [release_date, movie_name, title_id, year]

            except Exception as es:
                print("No! release data found on selected region"+"{0} :".format(type(es)), es)
                sys.exit(0)

        print("\rUpcoming Releases Extraction Completed\r", end="\r", flush=True)

# main class which passes the titleid to indiviual class
class imdb:
    def __init__(self):
        start_time = datetime.now()
        upcoming_releases.__init__(self)

        time_delta = datetime.now() - start_time
        print("\rCalculating time taken for upcoming releases extraction : %s  seconds" % (time_delta.seconds), end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb User Reviews
class user_reviews:

    """
    Collects user reviews of the multi media content in IMDb when titleid is given.
    :param user reviews: Unique identification titleid for every multi media content in IMDb.
    :returns: Returns all the user reviews.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.user_reviews_url = "https://www.imdb.com/title/" + str(self.titleid) + "/reviews?spoiler=hide&sort=helpfulnessScore&dir=desc&ratingFilter=0"

        # Creating soup for the website
        soup = BeautifulSoup(get(self.user_reviews_url).text, 'lxml')

        """
        :returns: movie title if available.
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        # for collection of number of reviews
        reviews_count = int(''.join(i for i in soup.select_one('div.header').span.text if i.isdigit()))

        maxclicks = int(reviews_count)//25

        options = Options()
        options.add_argument("--headless")
        browser = webdriver.Chrome(options=options)
        wait = WebDriverWait(browser, 100)
        browser.get(self.user_reviews_url)

        clicks = 0
        while True:
            clicks += 1
            if clicks <= maxclicks:
                more_button = wait.until(ec.visibility_of_element_located((By.CLASS_NAME, "ipl-load-more__button"))).click()
            else:
                break
            sys.stdout.write('\r' + str(clicks) + ' - ' + 'clicks has made for scrolling out of' + ' - ' + str(maxclicks))
            sys.stdout.flush()
        time.sleep(1)

        soup = BeautifulSoup(browser.page_source, 'lxml')
        browser.quit()

        user_reviews_count = soup.select('.text')
        self.user_reviews_count = len(user_reviews_count)
        sys.stdout.write('\r' + "\n" + str("total user reviews without spoilers are") + " : " + str(len(user_reviews_count)) + '\r')
        sys.stdout.flush()

        analyser = SentimentIntensityAnalyzer()
        neu_sum, neg_sum, compound_sum, pos_sum, count = 0,0,0,0,0

        self.user_reviews = [' '.join(item.text.split()) for item in user_reviews_count]
        sentiment_score = [analyser.polarity_scores(self.user_reviews[i])['compound'] for i in range(len(self.user_reviews))]
        polarity_score = [analyser.polarity_scores(self.user_reviews[i]) for i in range(len(self.user_reviews))]

        sentiment = []
        for item in user_reviews_count:
            analysis = TextBlob(' '.join(item.text.split()))
            # set sentiment
            if analysis.sentiment.polarity > 0:
                sentiment.append('positive')
            elif analysis.sentiment.polarity == 0:
                sentiment.append('neutral')
            else:
                sentiment.append('negative')

        self.user_reviews_df = pd.DataFrame({'Tweets' : self.user_reviews, 'Sentiment' : sentiment, 'Sentiment Score' : sentiment_score, 'Polarity Scorce' : polarity_score})

        for i in range(len(self.user_reviews)):
            count += 1
            score = analyser.polarity_scores(self.user_reviews[i])
            neu_sum += score['neu']
            neg_sum += score['neg']
            pos_sum += score['pos']

        if count:
            self.final_sentiment_scores = {"neu" : round(neu_sum / count, 3), "neg" : round(neg_sum / count, 3), "pos" : round(pos_sum / count, 3), "compound" : round(compound_sum / count, 3)}
        else:
            self.final_sentiment_scores = None

        sys.stdout.write('\r' + str("User Review Extraction Completed") +  '\r')
        sys.stdout.flush()

# main class which passes the titleid to indiviual class
class imdb:
    def __init__(self, titleid):
        start_time = datetime.now()
        user_reviews.__init__(self, titleid)

        time_delta = datetime.now() - start_time
        print("\rCalculating time taken for user review extraction : %s  seconds" % (time_delta.seconds), end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb Parental Guide Details
class parental_guide:

    """
    Collects IMDb Parental Guide Details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.parental_guide_url = "https://www.imdb.com/title/%s/parentalguide" % self.titleid
        soup = BeautifulSoup(get(self.parental_guide_url).text, 'lxml')

        """
        :returns: Movie Title
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        :returns: Certificate DataFrame if available.
        """
        try:
            certificate = soup.select_one('#certificates').select('tr')
            self.certificates_df = pd.DataFrame(columns=['Name', 'Type'])

            for i in range(len(certificate)):
                try:
                    self.certificates_df.loc[len(self.certificates_df)] = [certificate[i].select_one('td.ipl-zebra-list__label').text.strip(),
                                                               ' '.join(certificate[i].select_one('td').findNext('td').text.split())]
                except:
                    pass
        except:
            self.certificates_df = None

        """
        :returns: Certificate Name if available.
        """
        try:
            self.certificates_name = self.certificates_df.Name.tolist()
        except:
            self.certificates_name = None

        """
        :returns: Certificate Type if available.
        """
        try:
            self.certificates_type = self.certificates_df.Type.tolist()
        except:
            self.certificates_type = None

        """
        :returns: Adivsory Nudity status if available.
        """
        try:
            advisory_nudity = soup.select_one('#advisory-nudity').select('li.ipl-zebra-list__item')
            self.adivsory_nudity_status = str(soup.select_one('#advisory-nudity').select_one('.ipl-status-pill').text.strip()) + " : " + str(soup.select_one('#advisory-nudity').select_one('.advisory-severity-vote__message').text.strip())
        except:
            self.adivsory_nudity_status = None

        """
        :returns: Advisory Nudity Severity Vote if available.
        """
        try:
            advisory_nudity = soup.select_one('#advisory-nudity').select('li.ipl-zebra-list__item')
            self.advisory_nudity_severity_vote = [' '.join(advisory_nudity[i].text.split()).replace("\'", "") for i in range(len(advisory_nudity))]
        except:
            self.advisory_nudity_severity_vote = None

        """
        :returns: Adivsory Violence status if available.
        """
        try:
            advisory_violence = soup.select_one('#advisory-violence').select('li.ipl-zebra-list__item')
            self.advisory_violence_status = str(soup.select_one('#advisory-violence').select_one('.ipl-status-pill').text.strip()) + " : " + str(soup.select_one('#advisory-violence').select_one('.advisory-severity-vote__message').text.strip())
        except:
            self.advisory_violence_status = None

        """
        :returns: Advisory Violence Severity Vote if available.
        """
        try:
            advisory_violence = soup.select_one('#advisory-violence').select('li.ipl-zebra-list__item')
            self.advisory_violence_severity_vote = [' '.join(advisory_violence[i].text.split()).replace("\'", "") for i in range(len(advisory_violence))]
        except:
            self.advisory_violence_severity_vote = None

        """
        :returns: Adivsory Profanity status if available.
        """
        try:
            advisory_profanity = soup.select_one('#advisory-profanity').select('li.ipl-zebra-list__item')
            self.advisory_profanity_status = str(soup.select_one('#advisory-profanity').select_one('.ipl-status-pill').text.strip()) + " : " + str(soup.select_one('#advisory-profanity').select_one('.advisory-severity-vote__message').text.strip())
        except:
            self.advisory_profanity_status = None

        """
        :returns: Advisory Profanity Severity Vote if available.
        """
        try:
            advisory_profanity = soup.select_one('#advisory-profanity').select('li.ipl-zebra-list__item')
            self.advisory_profanity_severity_vote = [' '.join(advisory_profanity[i].text.split()).replace("\'", "") for i in range(len(advisory_profanity))]
        except:
            self.advisory_profanity_severity_vote = None

        """
        :returns: Adivsory Alcohol status if available.
        """
        try:
            advisory_alcohol = soup.select_one('#advisory-alcohol').select('li.ipl-zebra-list__item')
            self.advisory_alcohol_status = str(soup.select_one('#advisory-alcohol').select_one('.ipl-status-pill').text.strip()) + " : " + str(soup.select_one('#advisory-alcohol').select_one('.advisory-severity-vote__message').text.strip())
        except:
            self.advisory_alcohol_status = None

        """
        :returns: Advisory Alcohol Severity Vote if available.
        """
        try:
            advisory_alcohol = soup.select_one('#advisory-alcohol').select('li.ipl-zebra-list__item')
            self.advisory_alcohol_severity_vote = [' '.join(advisory_alcohol[i].text.split()).replace("\'", "") for i in range(len(advisory_alcohol))]
        except:
            self.advisory_alcohol_severity_vote = None

        """
        :returns: Adivsory Frightening status if available.
        """
        try:
            advisory_frightening = soup.select_one('#advisory-frightening').select('li.ipl-zebra-list__item')
            self.advisory_frightening_status = str(soup.select_one('#advisory-frightening').select_one('.ipl-status-pill').text.strip()) + " : " + str(soup.select_one('#advisory-frightening').select_one('.advisory-severity-vote__message').text.strip())
        except:
            self.advisory_frightening_status = None

        """
        :returns: Advisory Frightening Severity Vote if available.
        """
        try:
            advisory_frightening = soup.select_one('#advisory-frightening').select('li.ipl-zebra-list__item')
            self.advisory_frightening_severity_vote = [' '.join(advisory_frightening[i].text.split()).replace("\'", "") for i in range(len(advisory_frightening))]
        except:
            self.advisory_frightening_severity_vote = None

        """
        :returns: Spoilers Violence & Gore if available.
        """
        try:
            spoiler_violence = soup.select_one('#advisory-spoilers').select_one('#advisory-spoiler-violence').select('li.ipl-zebra-list__item')
            self.spoiler_violence = [' '.join(spoiler_violence[i].contents[0].split()) for i in range(len(spoiler_violence))]
        except:
            self.spoiler_violence = None

        """
        :returns: Spoilers Alcohol, Drugs & Smoking if available.
        """
        try:
            spoiler_alcohol = soup.select_one('#advisory-spoilers').select_one('#advisory-spoiler-alcohol').select('li.ipl-zebra-list__item')
            self.spoiler_alcohol = [' '.join(spoiler_alcohol[i].contents[0].split()) for i in range(len(spoiler_alcohol))]
        except:
            self.spoiler_alcohol = None

        """
        :returns: Spoilers Frightening & Intense Scenes if available.
        """
        try:
            spoiler_frightening = soup.select_one('#advisory-spoilers').select_one('#advisory-spoiler-frightening').select('li.ipl-zebra-list__item')
            self.spoiler_frightening = [' '.join(spoiler_frightening[i].contents[0].split()) for i in range(len(spoiler_frightening))]
        except:
            self.spoiler_frightening = None

        """
        :returns: Creates Dict from the above info. if available.
        """
        try:
            self.imdb_parental_guide_metadata = {"Movie Name" : self.title,
                                                 "Title ID" : self.titleid,
                                                 "Movie Parental Guide URL" : self.parental_guide_url,
                                                 "Certificate Name" : self.certificates_name,
                                                 "Certificate Type" : self.certificates_type,
                                                 "Sex & Nudity" : {"Nudity Status" : self.adivsory_nudity_status,
                                                                   "Nudity Severity" : self.advisory_nudity_severity_vote},
                                                 "Alcohol & Smoking" : {"Alcohol Status" : self.advisory_alcohol_status,
                                                                        "Alcohol Severity" : self.advisory_alcohol_severity_vote},
                                                 "Violence" : {"Violence" : self.advisory_violence_status,
                                                               "Violence Severity" : self.advisory_violence_severity_vote},
                                                 "Frighten" : {"Frighten Status" : self.advisory_frightening_status,
                                                               "Frighten Severity" : self.advisory_frightening_severity_vote},
                                                 "Profanity" : {"Profanity Status" : self.advisory_profanity_status,
                                                                "Profanity Severity" : self.advisory_profanity_severity_vote},
                                                 "Spoiler Violence" : self.spoiler_violence,
                                                 "Spoiler Alcohol" : self.spoiler_alcohol,
                                                 "Spoiler Frighten" : self.spoiler_frightening}
        except:
            self.imdb_parental_guide_metadata = None

        print("\rParental Guide Extraction Completed\r", end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb Plot Keywords Details
class plot_keywords:

    """
    Collects IMDb Plot Keywords Details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.plot_keywords_url = "https://www.imdb.com/title/%s/keywords" % self.titleid 
        soup = BeautifulSoup(get(self.plot_keywords_url).text, 'lxml')

        """
        :returns: Movie Title
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        :returns: Movie Plot Keywords
        """
        try:
            block = soup.select('td.soda')
            self.plot_keywords = [block[i]['data-item-keyword'] for i in range(len(block))]
        except:
            self.plot_keywords = None

        """
        :returns: Creates Dict from the above info. if available.
        """
        try:
            self.imdb_plot_Keywords_metadata = {"Movie Name" : self.title,
                                                "Title ID" : self.titleid,
                                                "Movie Plot Keywords URL" : self.plot_keywords_url,
                                                "Plot Keywords" : self.plot_keywords}
        except:
            self.imdb_plot_Keywords_metadata = None

        print("\rPlot Keywords Extraction Completed\r", end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb Technical Spec Details
class technical_spec:

    """
    Collects IMDb Technical Spec Details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.technical_spec_url = "https://www.imdb.com/title/%s/technical" % str(self.titleid)
        soup = BeautifulSoup(get(self.technical_spec_url).text, 'lxml')
        technical_spec = soup.select('td[class="label"]')

        """
        :returns: movie title if available.
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        :returns: movie runtime if available.
        """
        try:
            runtime_index = [i for i in range(len(soup.select('td[class="label"]'))) if 'runtime' in  ' '.join(soup.select('td[class="label"]')[i].text.split()).lower()][0]
            self.runtime = ' '.join(technical_spec[runtime_index].findNext('td').text.split())
        except:
            self.runtime = None

        """
        :returns: movie sound mix if available.
        """
        try:
            sound_mix_index = [i for i in range(len(technical_spec)) if 'sound mix' in  ' '.join(technical_spec[i].text.split()).lower()][0]
            self.sound_mix = ' '.join(technical_spec[sound_mix_index].findNext('td').text.split()).split('|')
        except:
            self.sound_mix = None

        """
        :returns: movie color if available.
        """
        try:
            color_index = [i for i in range(len(technical_spec)) if 'color' in  ' '.join(technical_spec[i].text.split()).lower()][0]
            self.color = ' '.join(technical_spec[color_index].findNext('td').text.split()).split('|')
        except:
            self.color = None

        """
        :returns: movie aspect ratio if available.
        """
        try:
            aspect_ratio_index = [i for i in range(len(technical_spec)) if 'aspect ratio' in  ' '.join(technical_spec[i].text.split()).lower()][0]
            aspect_ratio = list(technical_spec[aspect_ratio_index].findNext('td').stripped_strings)
            self.aspect_ratio = [' '.join(aspect_ratio[i].split()) for i in range(len(aspect_ratio))]
        except:
            self.aspect_ratio = None

        """
        :returns: movie camera if available.
        """
        try:
            camera_index = [i for i in range(len(technical_spec)) if 'camera' in  ' '.join(technical_spec[i].text.split()).lower()][0]
            camera = list(technical_spec[camera_index].findNext('td').stripped_strings)
            self.camera = [' '.join(camera[i].split()) for i in range(len(camera))]
        except:
            self.camera = None

        """
        :returns: movie laboratory if available.
        """
        try:
            laboratory_index = [i for i in range(len(technical_spec)) if 'laboratory' in  ' '.join(technical_spec[i].text.split()).lower()][0]
            laboratory = list(technical_spec[laboratory_index].findNext('td').stripped_strings)
            self.laboratory = [' '.join(laboratory[i].split()) for i in range(len(laboratory))]
        except:
            self.laboratory = None

        """
        :returns: negative format if available.
        """
        try:
            negative_format_index = [i for i in range(len(technical_spec)) if 'negative format' in  ' '.join(technical_spec[i].text.split()).lower()][0]
            self.negative_format = technical_spec[negative_format_index].findNext('td').text.strip()
        except:
            self.negative_format = None

        """
        :returns: cinematography process if available.
        """
        try:
            cinematographic_process_index = [i for i in range(len(technical_spec)) if 'cinematographic process' in  ' '.join(technical_spec[i].text.split()).lower()][0]
            cinematographic_process = list(technical_spec[cinematographic_process_index].findNext('td').stripped_strings)
            self.cinematographic_process = [' '.join(cinematographic_process[i].split()) for i in range(len(cinematographic_process))]
        except:
            self.cinematographic_process = None

        """
        :returns: printed film format if available.
        """
        try:
            printed_film_format_index = [i for i in range(len(technical_spec)) if 'printed film format' in  ' '.join(technical_spec[i].text.split()).lower()][0]
            printed_film_format = list(technical_spec[printed_film_format_index].findNext('td').stripped_strings)
            self.printed_film_format = [' '.join(printed_film_format[i].split()) for i in range(len(printed_film_format))]
        except:
            self.printed_film_format = None

        """
        :returns: film length if available.
        """
        try:
            film_length_index = [i for i in range(len(technical_spec)) if 'film length' in  ' '.join(technical_spec[i].text.split()).lower()][0]
            film_length = list(technical_spec[film_length_index].findNext('td').stripped_strings)
            self.film_length = [' '.join(film_length[i].split()) for i in range(len(film_length))]
        except:
            self.film_length = None

        """
        :returns: Creates Dict from the above info. if available.
        """
        try:
            self.imdb_technical_spec_metadata = {"Movie Name" : self.title,
                                                 "Title ID" : self.titleid,
                                                 "Movie Technical Spec URL" : self.technical_spec_url,
                                                 "Runtime" : self.runtime,
                                                 "Sound Mix" : self.sound_mix,
                                                 "Color" : self.color,
                                                 "Aspect Ratio" : self.aspect_ratio,
                                                 "Camera" : self.camera,
                                                 "Laboratory" : self.laboratory,
                                                 "Negative Film Format" : self.negative_format,
                                                 "Cinematography Process" : self.cinematographic_process,
                                                 "Printed Film Format" : self.printed_film_format,
                                                 "Film Length" : self.film_length}
        except:
            self.imdb_technical_spec_metadata = None

        print("\rTechincal Spec Extraction Completed\r", end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb Trending Now in India Details
class trending_now_in_india:

    """
    Collects trending_now_in_india details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self):
        base_url = "https://www.imdb.com/india/"

        self.upcoming_movies_url =  "%supcoming" % base_url
        self.popular_tamil_url = "%stamil" % base_url
        self.popular_telugu_url = "%stelugu" % base_url
        self.popular_india_url = "%sreleased" % base_url
        self.popular_global_url = "%sglobal" % base_url
        self.popular_hindi_url = "%shindi" % base_url

        upcoming_soup = BeautifulSoup(get(self.upcoming_movies_url).text, 'lxml')
        popular_tamil_soup = BeautifulSoup(get(self.popular_tamil_url).text, 'lxml')
        popular_telugu_soup = BeautifulSoup(get(self.popular_telugu_url).text, 'lxml')
        popular_india_soup = BeautifulSoup(get(self.popular_india_url).text, 'lxml')
        popular_global_soup = BeautifulSoup(get(self.popular_global_url).text, 'lxml')
        popular_global_soup = BeautifulSoup(get(self.popular_hindi_url).text, 'lxml')

        """
        :returns: Upcoming DataFrame
        """
        try:
            upcoming_movies = upcoming_soup.select_one('#trending-container').select('.trending-list-rank-item-data-container')
            self.upcoming_movies_df = pd.DataFrame(columns=['Rank', 'Name', 'ID', '% OF TOP 10 PAGE VIEWS'])

            for i in range(len(upcoming_movies)):
                try:
                    self.upcoming_movies_df.loc[len(self.upcoming_movies_df)] = [upcoming_movies[i].select_one('.trending-list-rank-item-rank-position').text,
                                                                                 upcoming_movies[i].select_one('.trending-list-rank-item-name').text,
                                                                                 upcoming_movies[i].select_one('.trending-list-rank-item-name').a['href'][7:16],
                                                                                 upcoming_movies[i].select_one('.trending-list-rank-item-share').text]
                except:
                    pass
        except:
            self.upcoming_movies_df = None

        """
        :returns: popular_tamil_movies DataFrame
        """
        try:
            popular_tamil_movies = popular_tamil_soup.select_one('#trending-container').select('.trending-list-rank-item-data-container')
            self.popular_tamil_movies_df = pd.DataFrame(columns=['S.No.', 'Name', 'ID', 'Share'])

            for i in range(len(popular_tamil_movies)):
                try:
                    self.popular_tamil_movies_df.loc[len(self.popular_tamil_movies_df)] = [popular_tamil_movies[i].select_one('.trending-list-rank-item-rank-position').text,
                                                                                           popular_tamil_movies[i].select_one('.trending-list-rank-item-name').text,
                                                                                           popular_tamil_movies[i].select_one('.trending-list-rank-item-name').a['href'][7:16],
                                                                                           popular_tamil_movies[i].select_one('.trending-list-rank-item-share').text]
                except:
                    pass
        except:
            self.popular_tamil_movies_df = None

        """
        :returns: popular_telugu_movies DataFrame
        """
        try:
            popular_telugu_movies = popular_telugu_soup.select_one('#trending-container').select('.trending-list-rank-item-data-container')
            self.popular_telugu_movies_df = pd.DataFrame(columns=['S.No.', 'Name', 'ID', 'Share'])

            for i in range(len(popular_telugu_movies)):
                try:
                    self.popular_telugu_movies_df.loc[len(self.popular_telugu_movies_df)] = [popular_telugu_movies[i].select_one('.trending-list-rank-item-rank-position').text,
                                                                                             popular_telugu_movies[i].select_one('.trending-list-rank-item-name').text,
                                                                                             popular_telugu_movies[i].select_one('.trending-list-rank-item-name').a['href'][7:16],
                                                                                             popular_telugu_movies[i].select_one('.trending-list-rank-item-share').text]
                except:
                    pass
        except:
            self.popular_telugu_movies_df = None

        """
        :returns: Popular India DataFrame
        """
        try:
            popular_india_movies = popular_india_soup.select_one('#trending-container').select('.trending-list-rank-item-data-container')
            self.popular_india_movies_df = pd.DataFrame(columns=['Rank', 'Name', 'ID', '% OF TOP 10 PAGE VIEWS'])

            for i in range(len(popular_india_movies)):
                try:
                    self.popular_india_movies_df.loc[len(self.popular_india_movies_df)] = [popular_india_movies[i].select_one('.trending-list-rank-item-rank-position').text,
                                                                                           popular_india_movies[i].select_one('.trending-list-rank-item-name').text,
                                                                                           popular_india_movies[i].select_one('.trending-list-rank-item-name').a['href'][7:16],
                                                                                           popular_india_movies[i].select_one('.trending-list-rank-item-share').text]
                except:
                    pass
        except:
            self.popular_india_movies_df = None

        """
        :returns: Poppular Global DataFrame
        """
        try:
            popular_global_movies = popular_global_soup.select_one('#trending-container').select('.trending-list-rank-item-data-container')
            self.popular_global_movies_df = pd.DataFrame(columns=['Rank', 'Name', 'ID', '% OF TOP 10 PAGE VIEWS'])

            for i in range(len(popular_global_movies)):
                try:
                    self.popular_global_movies_df.loc[len(self.popular_global_movies_df)] = [popular_global_movies[i].select_one('.trending-list-rank-item-rank-position').text,
                                                                                             popular_global_movies[i].select_one('.trending-list-rank-item-name').text,
                                                                                             popular_global_movies[i].select_one('.trending-list-rank-item-name').a['href'][7:16],
                                                                                             popular_global_movies[i].select_one('.trending-list-rank-item-share').text]
                except:
                    pass
        except:
            self.popular_global_movies_df = None

        """
        :returns: Poppular Hindi DataFrame
        """
        try:
            popular_hindi_movies = popular_global_soup.select_one('#trending-container').select('.trending-list-rank-item-data-container')
            self.popular_hindi_movies_df = pd.DataFrame(columns=['Rank', 'Name', 'ID', '% OF TOP 10 PAGE VIEWS'])

            for i in range(len(popular_hindi_movies)):
                try:
                    self.popular_hindi_movies_df.loc[len(self.popular_hindi_movies_df)] = [popular_hindi_movies[i].select_one('.trending-list-rank-item-rank-position').text,
                                                                                           popular_hindi_movies[i].select_one('.trending-list-rank-item-name').text,
                                                                                           popular_hindi_movies[i].select_one('.trending-list-rank-item-name').a['href'][7:16],
                                                                                           popular_hindi_movies[i].select_one('.trending-list-rank-item-share').text]
                except:
                    pass
        except:
            self.popular_hindi_movies_df = None

        print("\rTrending Now in India Extraction Completed\r", end="\r", flush=True)

# main class which passes the titleid to indiviual class
class imdb:
    def __init__(self):
        start_time = datetime.now()
        trending_now_in_india.__init__(self)

        time_delta = datetime.now() - start_time
        print("\rCalculating time taken for trending now in india extraction : %s  seconds\r" % (time_delta.seconds), end="\r", flush=True)



from imdby.cast_and_crew import cast_and_crew
from imdby.company import company
from imdby.movie import movie
from imdby.parental_guide import parental_guide
from imdby.plot import plot
from imdby.plot_keywords import plot_keywords
from imdby.release_info import release_info
from imdby.taglines import taglines
from imdby.technical_spec import technical_spec
from imdby.utils import *


# main class which passes the titleid to each indiviual class
class imdb:
    def __init__(self, titleid):
        start_time = datetime.now()
        movie.__init__(self, titleid)
        plot.__init__(self, titleid)
        plot_keywords.__init__(self, titleid)
        parental_guide.__init__(self, titleid)
        company.__init__(self, titleid)
        technical_spec.__init__(self, titleid)
        release_info.__init__(self, titleid)
        taglines.__init__(self, titleid)
        cast_and_crew.__init__(self, titleid)

        time_delta = datetime.now() - start_time
        print("\rCalculating time taken for imdb basic info. extraction : %s  seconds" % (time_delta.seconds), end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb Full Cast & Crew Details
class cast_and_crew:

    """
    Collects full cast & crew details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """
    def __init__(self, titleid):
        self.titleid = titleid
        self.cast_and_crew_url = "https://www.imdb.com/title/%s/fullcredits" % self.titleid
        soup = BeautifulSoup(get(self.cast_and_crew_url).text, 'lxml')

        """
        :returns: Movie Title
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        :returns: Writtern Credits DataFrame
        """
        try:
            w_index = [i for i in range(len(soup.select('h4'))) if 'writing' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            writer = soup.select('h4')[w_index].findNext('table').select('tr')
            self.writers_df = pd.DataFrame(columns=['Name', 'Credit', 'ID'])

            for i in range(len(writer)):
                try:
                    self.writers_df.loc[len(self.writers_df)] = [writer[i].select_one('td.name').a.text.strip(),
                                                                 writer[i].select_one('td.credit').text.strip(),
                                                                 writer[i].select_one('td.name').a['href'][6:15].strip()]
                except:
                    pass
        except:
            self.writers_df = None

        """
        :returns: Writers list
        """
        try:
            self.writers = self.writers_df.Name.tolist()
        except:
            self.writers = None

        """
        :returns: Writers ID list
        """
        try:
            self.writers_id = self.writers_df.ID.tolist()
        except:
            self.writers_id = None

        """
        :returns: Writers Credit list
        """
        try:
            self.writers_credit = self.writers_df.Credit.tolist()
        except:
            self.writers_credit = None

        """
        :returns: Directed_by DataFrame
        """
        try:
            d_index = [i for i in range(len(soup.select('h4'))) if 'directed' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            director = soup.select('h4')[d_index].findNext('table').select('tr')
            self.directors_df = pd.DataFrame(columns=['Name', 'ID'])

            for i in range(len(director)):
                try:
                    self.directors_df.loc[len(self.directors_df)] = [director[i].select_one('td.name').a.text.strip(),
                                                                     director[i].select_one('td.name').a['href'][6:15].strip()]
                except:
                    pass
        except:
            self.directors_df = None

        """
        :returns: Directors list
        """
        try:
            self.directors = self.directors_df.Name.tolist()
        except:
            self.directors = None

        """
        :returns: Directors ID list
        """
        try:
            self.directors_id = self.directors_df.ID.tolist()
        except:
            self.directors_id = None

        """
        :returns: Produced_by DataFrame
        """
        try:
            p_index = [i for i in range(len(soup.select('h4'))) if 'produced' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            producer = soup.select('h4')[p_index].findNext('table').select('tr')
            self.producers_df = pd.DataFrame(columns=['Name', 'Credit', 'ID'])

            for i in range(len(producer)):
                try:
                    self.producers_df.loc[len(self.producers_df)] = [producer[i].select_one('td.name').a.text.strip(),
                                                                     producer[i].select_one('td.credit').text.strip(),
                                                                     producer[i].select_one('td.name').a['href'][6:15].strip()]
                except:
                    pass
        except:
            self.producers_df = None

        """
        :returns: Producers list
        """
        try:
            self.producers = self.producers_df.Name.tolist()
        except:
            self.producers = None

        """
        :returns: Producers ID list
        """
        try:
            self.producers_id = self.producers_df.ID.tolist()
        except:
            self.producers_id = None

        """
        :returns: Producers Credit list
        """
        try:
            self.producers_credit = self.producers_df.Credit.tolist()
        except:
            self.producers_credit = None

        """
        :returns: Cast DataFrame
        """
        try:
            c_index = [i for i in range(len(soup.select('h4'))) if 'cast' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            casting = soup.select('h4')[c_index].findNext('table').select('tr')
            self.cast_df = pd.DataFrame(columns=['Name', 'Image', 'Character', 'ID'])

            for i in range(len(casting)):
                try:
                    self.cast_df.loc[len(self.cast_df)] = [casting[i].select_one('td.primary_photo').a.img['title'].strip(),
                                                           casting[i].select_one('td.primary_photo').a.img['src'].strip(),
                                                           ' '.join(casting[i].select_one('td.character').text.split()).strip(),
                                                           casting[i].select_one('td.primary_photo').a['href'][6:15].strip()]
                except:
                    pass
        except:
            self.cast_df = None

        """
        :returns: Cast list
        """
        try:
            self.cast = self.cast_df.Name.tolist()
        except:
            self.cast = None

        """
        :returns: Cast ID list
        """
        try:
            self.cast_id = self.cast_df.ID.tolist()
        except:
            self.cast_id = None

        """
        :returns: Cast Character/Role list
        """
        try:
            self.cast_character = self.cast_df.Character.tolist()
        except:
            self.cast_character = None

        """
        :returns: Cast Image URL list
        """
        try:
            self.cast_character_img_url = self.cast_df.Image.tolist()
        except:
            self.cast_character_img_url = None

        """
        :returns: Music by DataFrame
        """
        try:
            m_index = [i for i in range(len(soup.select('h4'))) if 'music' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            music = soup.select('h4')[m_index].findNext('table').select('tr')
            self.music_df = pd.DataFrame(columns=['Name', 'ID'])

            for i in range(len(music)):
                try:
                    self.music_df.loc[len(self.music_df)] = [music[i].select_one('td.name').a.text.strip(),
                                                             music[i].select_one('td.name').a['href'][6:15].strip()]
                except:
                    pass
        except:
            self.music_df = None

        """
        :returns: Music by list
        """
        try:
            self.music = self.music_df.Name.tolist()
        except:
            self.music = None

        """
        :returns: Music by ID list
        """
        try:
            self.music_id = self.music_df.ID.tolist()
        except:
            self.music_id = None

        """
        :returns: Cinematography by DataFrame
        """
        try:
            ct_index = [i for i in range(len(soup.select('h4'))) if 'cinematography' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            cinematography = soup.select('h4')[ct_index].findNext('table').select('tr')
            self.cinematography_df = pd.DataFrame(columns=['Name', 'ID'])

            for i in range(len(cinematography)):
                try:
                    self.cinematography_df.loc[len(self.cinematography_df)] = [cinematography[i].select_one('td.name').a.text.strip(),
                                                                               cinematography[i].select_one('td.name').a['href'][6:15].strip()]
                except:
                    pass
        except:
            self.cinematography_df = None

        """
        :returns: Cinematography by list
        """
        try:
            self.cinematography = self.cinematography_df.Name.tolist()
        except:
            self.cinematography = None

        """
        :returns: Cinematography by ID list
        """
        try:
            self.cinematography_id = self.cinematography_df.ID.tolist()
        except:
            self.cinematography_id = None

        """
        :returns: Production Design by DataFrame
        """
        try:
            pd_index = [i for i in range(len(soup.select('h4'))) if 'production design' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            production_designer = soup.select('h4')[pd_index].findNext('table').select('tr')
            self.production_designer_df = pd.DataFrame(columns=['Name', 'ID'])

            for i in range(len(production_designer)):
                try:
                    self.production_designer_df.loc[len(self.production_designer_df)] =[production_designer[i].select_one('td.name').a.text.strip(),
                                                                                       production_designer[i].select_one('td.name').a['href'][6:15].strip()]
                except:
                    pass
        except:
            self.cinematography_df = None

        """
        :returns: Production Design by list
        """
        try:
            self.production_designer = self.production_designer_df.Name.tolist()
        except:
            self.production_designer = None

        """
        :returns: Production Design by ID list
        """
        try:
            self.production_designer_id = self.production_designer_df.ID.tolist()
        except:
            self.production_designer_id = None

        """
        :returns: Film Editing by  DataFrame
        """
        try:
            fe_index = [i for i in range(len(soup.select('h4'))) if 'film editing' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            film_editing = soup.select('h4')[fe_index].findNext('table').select('tr')
            self.film_editing_df = pd.DataFrame(columns=['Name', 'Credit', 'ID'])

            for i in range(len(film_editing)):
                try:
                    self.film_editing_df.loc[len(self.film_editing_df)] = [film_editing[i].select_one('td.name').a.text.strip(),
                                                                           film_editing[i].select_one('td.credit').text.strip(),
                                                                           film_editing[i].select_one('td.name').a['href'][6:15].strip()]
                except:
                    pass
        except:
            self.film_editing_df = None

        """
        :returns: Film Editing list
        """
        try:
            self.film_editing = self.film_editing_df.Name.tolist()
        except:
            self.film_editing = None

        """
        :returns: Film Editing ID list
        """
        try:
            self.film_editing_id = self.film_editing_df.ID.tolist()
        except:
            self.film_editing_id = None

        """
        :returns: Film Editing Credit list
        """
        try:
            self.film_editing_credit = self.film_editing_df.Credit.tolist()
        except:
            self.film_editing_credit = None

        """
        :returns: Casting by  DataFrame
        """
        try:
            cg_index = [i for i in range(len(soup.select('h4'))) if 'casting' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            casting_d = soup.select('h4')[cg_index].findNext('table').select('tr')
            self.casting_df = pd.DataFrame(columns=['Name', 'Credit', 'ID'])

            for i in range(len(casting_d)):
                try:
                    self.casting_df.loc[len(self.casting_df)] = [casting_d[i].select_one('td.name').a.text.strip(),
                                                                 casting_d[i].select_one('td.credit').text.strip(),
                                                                 casting_d[i].select_one('td.name').a['href'][6:15].strip()]
                except:
                    pass
        except:
            self.casting_df = None

        """
        :returns: Casting list
        """
        try:
            self.casting = self.casting_df.Name.tolist()
        except:
            self.casting = None

        """
        :returns: Casting ID list
        """
        try:
            self.casting_id = self.casting_df.ID.tolist()
        except:
            self.casting_id = None

        """
        :returns: Casting Credit list
        """
        try:
            self.casting_credit = self.casting_df.Credit.tolist()
        except:
            self.casting_credit = None

        """
        :returns: Creates Dict from the above info. if available.
        """
        try:
            self.imdb_cast_metadata = {"Movie Name" : self.title,
                                       "Title ID" : self.titleid,
                                       "Movie Full Cast and Crew URL" : self.cast_and_crew_url,
                                       "Director" : { "Name": self.directors,
                                                     "ID" : self.directors_id},
                                       "Writer" : {"Name" : self.writers,
                                                   "Credit" : self.writers_credit,
                                                   "ID" : self.writers_id},
                                       "Cast" : {"Name" : self.cast,
                                                 "Character" : self.cast_character,
                                                 "Image" : self.cast_character_img_url,
                                                 "ID" : self.cast_id},
                                       "Producer" : {"Name" : self.producers,
                                                     "Credit" : self.producers_credit,
                                                     "ID" : self.producers_id},
                                       "Music" : {"Name" : self.music,
                                                  "ID" : self.music_id},
                                       "Cinematography" : {"Name" : self.cinematography,
                                                           "ID" : self.cinematography_id},
                                       "Production Desing" : {"Name" : self.production_designer,
                                                              "ID" : self.production_designer_id},
                                       "Flim Editing" : {"Name" : self.film_editing,
                                                         "Credit" : self.film_editing_credit,
                                                         "ID" : self.film_editing_id},
                                       "Casting" : {"Name" : self.casting,
                                                    "Credit" : self.casting_credit,
                                                    "ID" : self.casting_id}}
        except:
            self.imdb_cast_metadata = None

        print("\rFull Cast & Crew Extraction Completed\r", end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb MovieDetails
class movie:

    """
    Collects basic movie details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.movie_url = "https://www.imdb.com/title/%s" % str(self.titleid)
        soup = BeautifulSoup(get(self.movie_url).text, 'lxml')

        """
        :returns: Movie Name if available.
        """
        try:
            self.title = soup.select_one('h1[class=""]').contents[0].strip()
        except:
            self.title = None

        """
        :returns: Genre types for the movie if available.
        """
        try:
            self.genre = [soup.find('h4', string='Genres:').findNextSiblings('a')[i].text.strip() for i in range(len(soup.find('h4', string='Genres:').findNextSiblings('a')))]
        except:
            self.genre = None

        """
        :returns: IMDb rating for the movie if available.
        """
        try:
            self.rating = soup.select_one('span[itemprop="ratingValue"]').text.strip()
        except:
            self.rating = None

        """
        :returns: IMDb votes obtained for the movie if available.
        """
        try:
            self.votes = soup.select_one('span[itemprop="ratingCount"]').text.strip()
        except:
            self.votes = None

        """
        :returns: Metascore of the movie if available.
        """
        try:
            self.metascore = soup.select_one('.metacriticScore').text.replace(u'\xa0', u' ').strip()
        except:
            self.metascore = None

        """
        :returns: Storyline of the movie if available.
        """
        try:
            self.storyline = soup.select_one('.summary_text').text.strip()
        except:
            self.storyline = None

        """
        :returns: Budget of the movie if available.
        """
        try:
            self.budget = soup.find('h4', string='Budget:').nextSibling.replace(u'\xa0', u' ').strip()
        except:
            self.budget = None

        """
        :returns: Opening Weekend USA of the movie if available.
        """
        try:
            self.opening_weekend_usa = soup.find('h4', string='Opening Weekend USA:').nextSibling.replace(u'\xa0', u' ').strip()
        except:
            self.opening_weekend_usa = None

        """
        :returns: Gross USA of the movie if available.
        """
        try:
            self.gross_usa = soup.find('h4', string='Gross USA:').nextSibling.replace(u'\xa0', u' ').strip()
        except:
            self.gross_usa = None

        """
        :returns: Cumulative Worldwide Gross of the movie if available.
        """
        try:
            self.cumulative_worldwide_gross = soup.find('h4', string='Cumulative Worldwide Gross:').nextSibling.replace(u'\xa0', u' ').strip()
        except:
            self.cumulative_worldwide_gross = None

        """
        :returns: Movie Poster URL if available.
        """
        try:
            self.movie_poster_url = 'https://www.imdb.com%s' % soup.select_one('.poster').a['href']
        except:
            self.movie_poster_url = None

        """
        :returns: Movie Released Year if available.
        """
        try:
            self.movie_release_year = soup.select_one('span[id="titleYear"]').a.text.strip()
        except:
            self.movie_release_year = None

        """
        :returns: Creates Meta Data from the above info. if available.
        """
        try:
            self.imdb_movie_metadata = {"Movie Name" : self.title,
                                        "Title ID" : self.titleid,
                                        "Rating" : self.rating,
                                        "IMDb Votes" : self.votes,
                                        "Genre" : self.genre,
                                        "Year" : self.movie_release_year,
                                        "Metascore" : self.metascore,
                                        "Movie Poster URL" : self.movie_poster_url,
                                        "Budget" : self.budget,
                                        "Opening Weekend USA" : self.opening_weekend_usa,
                                        "Cumulative Worldwide Gross" : self.cumulative_worldwide_gross,
                                        "Gross USA" : self.gross_usa,
                                        "Storyline" : self.storyline,
                                        "Movie URL" : self.movie_url}
        except:
            self.imdb_movie_metadata = None

        print("\rBasic Movie Info Extraction Completed\r", end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb Company Credits Details
class company:

    """
    Collects Company Credits details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """
    def __init__(self, titleid):
        self.titleid = titleid
        self.company_url = "https://www.imdb.com/title/%s/companycredits" % self.titleid 
        soup = BeautifulSoup(get(self.company_url).text, 'lxml')

        """
        :returns: Movie Title
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        :returns: Production Company Credits DataFrame
        """
        try:
            production_company = soup.select_one('#production').findNext('ul').select('li')
            self.production_company_df = pd.DataFrame(columns=['Name', 'ID'])

            for i in range(len(production_company)):
                try:
                    self.production_company_df.loc[len(self.production_company_df)] = [production_company[i].a.text.strip(),
                                                                                       production_company[i].a['href'][9:].strip()]
                except:
                    pass
        except:
            self.production_company_df = None

        """
        :returns: Production Company Credits Name if available.
        """
        try:
            self.production_company_name = self.production_company_df.Name.tolist()
        except:
            self.production_company_name = None

        """
        :returns: Production Company Credits ID if available.
        """
        try:
            self.production_company_id = self.production_company_df.ID.tolist()
        except:
            self.production_company_id = None
        """
        :returns: Distributors DataFrame
        """
        try:
            distributors = soup.select_one('#distributors').findNext('ul').select('li')
            self.distributors_df = pd.DataFrame(columns=['Name', 'ID'])

            for i in range(len(distributors)):
                try:
                    self.distributors_df.loc[len(self.distributors_df)] = [distributors[i].a.text.strip(),
                                                                           distributors[i].a['href'][9:].strip()]
                except:
                    pass
        except:
            self.distributors_df = None

        """
        :returns: Distributors Name if available.
        """
        try:
            self.distributors_name = self.distributors_df.Name.tolist()
        except:
            self.distributors_name = None

        """
        :returns: Distributors ID if available.
        """
        try:
            self.distributors_id = self.distributors_df.ID.tolist()
        except:
            self.distributors_id = None

        """
        :returns: Special Effects DataFrame
        """
        try:
            special_effects = soup.select_one('#specialEffects').findNext('ul').select('li')
            self.specials_effects_df = pd.DataFrame(columns=['Name', 'ID'])

            for i in range(len(special_effects)):
                try:
                    self.specials_effects_df.loc[len(self.specials_effects_df)] = [special_effects[i].a.text.strip(),
                                                                                   special_effects[i].a['href'][9:].strip()]
                except:
                    pass
        except:
            self.specials_effects_df = None

        """
        :returns: Special Effects Name if available.
        """
        try:
            self.specials_effects_name = self.specials_effects_df.Name.tolist()
        except:
            self.specials_effects_name = None

        """
        :returns: Special Effects ID if available.
        """
        try:
            self.specials_effects_id = self.specials_effects_df.ID.tolist()
        except:
            self.specials_effects_id = None

        """
        :returns: Other Companies DataFrame
        """
        try:
            other_companies = soup.select_one('#other').findNext('ul').select('li')
            self.other_companies_df = pd.DataFrame(columns=['Name', 'ID'])

            for i in range(len(other_companies)):
                try:
                    self.other_companies_df.loc[len(self.other_companies_df)] = [other_companies[i].a.text.strip(),
                                                                                  other_companies[i].a['href'][9:].strip()]
                except:
                    pass
        except:
            self.other_companies_df = None

        """
        :returns: Other Companies Name if available.
        """
        try:
            self.other_companies_name = self.other_companies_df.Name.tolist()
        except:
            self.other_companies_name = None

        """
        :returns: Other Companies ID if available.
        """
        try:
            self.other_companies_id = self.other_companies_df.ID.tolist()
        except:
            self.other_companies_id = None

        """
        returns: Creates Dict from the above info. if available.
        """
        try:
            self.imdb_company_metadata = {"Movie Name" : self.title,
                                                 "Title ID" : self.titleid,
                                                 "Movie Company URL" : self.company_url,
                                                 "Production Company" : {"Name" : self.production_company_name,
                                                                         "ID" : self.production_company_id},
                                                 "Special Effects" : {"Name" : self.specials_effects_name,
                                                                      "ID" : self.specials_effects_id},
                                                 "Distributors" : {"Name" : self.distributors_name,
                                                                   "ID" : self.distributors_id},
                                                 "Other Companies" : {"Name" : self.other_companies_name,
                                                                      "ID" : self.other_companies_id}}
        except:
            self.imdb_company_metadata = None

        print("\rCompany Credits Extraction Completed\r", end="\r", flush=True)



from imdby.utils import *


# Retrieves External Sites Details
class external_sites:

    """
    Collects External Sites details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.external_sites_url = "https://www.imdb.com/title/%s/externalsites" % self.titleid 
        soup = BeautifulSoup(get(self.external_sites_url).text, 'lxml')

        """
        returns: Movie Title
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        returns: Official Sites DataFrame if available.
        """
        try:
            titles, urls = [], []
            official_sites = soup.select_one('#official').findNext('ul').select('li')
            for item in official_sites:
                try:
                    titles.append(item.text.strip())
                    urls.append(get('https://www.imdb.com%s' % item.a['href']).url)
                except:
                    titles.append(None)
                    urls.append(None)
            self.official_sites_df = pd.DataFrame({'Name' : titles, 'URL' : urls})
        except:
            self.official_sites_df = None

        """
        :returns: Official Site Name list if available.
        """
        try:
            self.official_sites_names = self.official_sites_df.Name.tolist()
        except:
            self.official_sites_names = None

        """
        :returns: Official Site URLs list if available.
        """
        try:
            self.official_sites_urls = self.official_sites_df.URL.tolist()
        except:
            self.official_sites_urls = None

        """
        returns: Miscellaneous Sites DataFrame if available.
        """
        try:
            titles, urls = [], []
            miscellaneous_sites = soup.select_one('#misc').findNext('ul').select('li')
            for item in miscellaneous_sites:
                try:
                    titles.append(item.text.strip())
                    urls.append(get('https://www.imdb.com%s' % item.a['href']).url)
                except:
                    titles.append(None)
                    urls.append(None)
            self.miscellaneous_sites_df = pd.DataFrame({'Name' : titles, 'URL' : urls})
        except:
            self.miscellaneous_sites_df = None

        """
        :returns: Miscellaneous Sites Names list if available.
        """
        try:
            self.miscellaneous_sites_names = self.miscellaneous_sites_df.Name.tolist()
        except:
            self.miscellaneous_sites_names = None

        """
        :returns: Miscellaneous Sites URLs list if available.
        """
        try:
            self.miscellaneous_sites_urls = self.miscellaneous_sites_df.URL.tolist()
        except:
            self.miscellaneous_sites_urls = None

        """
        returns: Photographs Sites DataFrame if available.
        """
        try:
            titles, urls = [], []
            photographs_sites = soup.select_one('#photos').findNext('ul').select('li')
            for item in photographs_sites:
                try:
                    titles.append(item.text.strip())
                    urls.append(get('https://www.imdb.com%s' % item.a['href']).url)
                except:
                    titles.append(None)
                    urls.append(None)
            self.photographs_sites_df = pd.DataFrame({'Title' : titles, 'URL' : urls})
        except:
            self.photographs_sites_df = None

        """
        :returns: Photographs Sites Names list if available.
        """
        try:
            self.photographs_sites_names = self.photographs_sites_df.Name.tolist()
        except:
            self.photographs_sites_names = None

        """
        :returns: Photographs Sites URLs list if available.
        """
        try:
            self.photographs_sites_urls = self.photographs_sites_df.URL.tolist()
        except:
            self.photographs_sites_urls = None

        """
        returns: Videos Clips and Trailers Sites DataFrame if available.
        """
        try:
            titles, urls = [], []
            videos_clips_and_trailers_sites = soup.select_one('#videos').findNext('ul').select('li')
            for item in videos_clips_and_trailers_sites:
                try:
                    titles.append(item.text.strip())
                    urls.append(get('https://www.imdb.com%s' % item.a['href']).url)
                except:
                    titles.append(None)
                    urls.append(None)
            self.videos_clips_and_trailers_sites_df = pd.DataFrame({'Title' : titles, 'URL' : urls})
        except:
            self.videos_clips_and_trailers_sites_df = None

        """
        :returns: Videos Clips and Trailers Sites Names list if available.
        """
        try:
            self.videos_clips_and_trailers_sites_names = self.videos_clips_and_trailers_sites_df.Name.tolist()
        except:
            self.videos_clips_and_trailers_sites_names = None

        """
        :returns: Videos Clips and Trailers Sites URLs list if available.
        """
        try:
            self.videos_clips_and_trailers_sites_urls = self.videos_clips_and_trailers_sites_df.URL.tolist()
        except:
            self.videos_clips_and_trailers_sites_urls = None

        """
        :returns: Creates Meta Data from the above info. if available.
        """
        try:
            self.imdb_external_sites_metadata = {"Movie Title" : self.title,
                                               "Title ID" : self.titleid,
                                               "Movie URL" : self.external_sites_url,
                                               "Official Sites" : {"Name" : self.official_sites_names,
                                                                  "URL" : self.official_sites_urls},
                                               "Miscellaneous Sites" : {"Name" : self.miscellaneous_sites_names,
                                                                        "URL" : self.miscellaneous_sites_urls},
                                               "Photographs" : {"Name" : self.photographs_sites_names,
                                                                "URL" : self.photographs_sites_urls},
                                               "Video Clips and Trailers" : {"Name" : self.videos_clips_and_trailers_sites_names,
                                                                             "URL" : self.videos_clips_and_trailers_sites_urls}}
        except:
            self.imdb_external_sites_metadata = None

        sys.stdout.write('\r' + str("External Sites Extraction Completed") +  '\r')
        sys.stdout.flush()

# main class which passes the titleid to indiviual class
class imdb:
    def __init__(self, titleid):
        start_time = datetime.now()
        external_sites.__init__(self, titleid)

        time_delta = datetime.now() - start_time
        print("\rCalculating time taken for external sites extraction : %s  seconds\r" % (time_delta.seconds), end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb Taglines
class taglines:

    """
    Collects tagline details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.taglines_url = "https://www.imdb.com/title/" + str(self.titleid) + "/taglines"
        soup = BeautifulSoup(get(self.taglines_url).text, 'lxml')
        taglines = soup.select_one('#taglines_content').select('.soda')

        """
        returns: Movie Title if available
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        returns: taglines if available
        """
        try:
            self.taglines = [' '.join(taglines[i].text.split()) for i in range(len(taglines))]
        except:
            self.taglines = None

        """
        :returns: Creates Dict from the above info. if available.
        """
        try:
            self.imdb_taglines_metadata = {"Movie Title" : self.title,
                                           "Title ID" : self.titleid,
                                           "Movie Taglines URL" : self.taglines_url,
                                           "Taglines" : self.taglines}
        except:
            self.imdb_taglines_metadata = None

        print("\rTaglines Extraction Completed\r", end="\r", flush=True)



from imdby.utils import *


class person_name:
    """
    'IMDbID' method takes a person name, searches for the similar person names available in IMDb.
    and suggests the person name to the user. Based upon user's selection it returns a list of person ids.

    Input serial numbers separated with spaces.
    -------------------- -------------------- -------------------- -------------------- --------------------
    :param movie: person name.
    :returns: A list of person ids.
    :raises ValueError: Raises an exception when entered a charater other than a number for selection.
    :raises IndexError: Raises an BeautifulSoup exception when index is out of range.
    """

    def __init__(self, text):
        self.entered_text = text
        self.url = 'https://www.imdb.com/find?q=%s&s=nm&ref_=fn_al_nm_mr' % ''.join(self.entered_text.split())
        soup = BeautifulSoup(get(self.url).text, 'lxml')

        suggestions, names, imdbids = [], [], []

        name = soup.select('.result_text')

        try:
            if len(name) > 20:
                for item in name[:20]:
                    print('%s' %(name.index(item) + 1) + ': ' + item.text.strip())
                    suggestions.append(item.a['href'][6:15])
                    names.append(item.a.text.strip())
            else:
                for item in name:
                    print('%s' %(name.index(item) + 1) + ': ' + item.text.strip())
                    suggestions.append(item.a['href'][6:15])
                    names.append(item.a.text.strip())

            input_name = re.findall(r"[\w']+", input('Enter serial number\t'))
            imdbids = [suggestions[int(load) - 1] if int(load) != 0 else '' for load in input_name]
            names = [names[int(load) - 1] if int(load) != 0 else '' for load in input_name]


            if len(imdbids) == 1:
                self.person_name = names[0]
                self.person_id = imdbids[0]
            else:
                self.person_name = names
                self.person_id = imdbids

        except Exception as es:
            print("{0} :".format(type(es)), es)
            sys.exit(0)

# main class which passes the titleid to indiviual class
class imdb:
    def __init__(self, text):
        start_time = datetime.now()
        person_name.__init__(self, text)

        time_delta = datetime.now() - start_time
        print("\rCalculating time taken for search person extraction : %s  second\r" % (time_delta.seconds), end="\r", flush=True)



"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/santhoshse7en/imdby
"""
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

# Always prefer setuptools over distutils
import setuptools

keywords = ['imdby', 'imdb', 'movie', 'people', 'database', 'cinema', 'film', 'person',
            'cast', 'actor', 'actress', 'director', 'sql', 'character',
            'company', 'package', 'plain text data files',
            'keywords', 'top250', 'bottom100', 'xml', 'bs4']

setuptools.setup(
    name="imdby",
    version="0.1.5",
    python_requires='>=3.5',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    author="M Santhosh Kumar",
    author_email="santhoshse7en@gmail.com",
    description="Python package to access the IMDb's database",
    url="https://github.com/santhoshse7en/imdby",
    platforms = 'any',
    keywords = keywords,
    install_requires=['beautifulsoup4', 'pandas', 'selenium', 'vaderSentiment', 'textblob', 'chromedriver-binary'],
    packages=setuptools.find_packages(),
    classifiers=['Development Status :: 4 - Beta',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Communications :: Email',
          'Topic :: Office/Business',
          'Topic :: Software Development :: Bug Tracking',
          ],
)



from imdby.utils import *


# Retrieves IMDb Release Info
class release_info:

    """
    Collects Release Info details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.release_info_url = "https://www.imdb.com/title/%s/releaseinfo" % self.titleid 
        soup = BeautifulSoup(get(self.release_info_url).text, 'lxml')

        """
        returns: Movie Title
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        returns: Release Info DataFrame if available.
        """
        releases_index = [i for i in range(len(soup.select('h4'))) if 'release' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
        releases = soup.select('h4')[releases_index].findNext('table').select('tr')
        country, date, location = [], [], []
        try:
            for i in range(len(releases)):
                try:
                    country.append(releases[i].select_one('td.release-date-item__country-name').a.text.strip())
                except:
                    country.append(None)

                try:
                    date.append(releases[i].select_one('td.release-date-item__date').text.strip())
                except:
                    date.append(None)

                try:
                    location.append(releases[i].select_one('td.release-date-item__attributes').text.strip())
                except:
                    location.append(None)

                self.releases_df = pd.DataFrame({'Country' : country, 'Date' : date, 'Location' : location})
        except:
            self.releases_df = None

        """
        :returns: Released Countries list if available.
        """
        try:
            self.released_country_names = self.releases_df.Country.tolist()
        except:
            self.released_country_names = None

        """
        :returns: Released Dates list if available.
        """
        try:
            self.released_dates = self.releases_df.Date.tolist()
        except:
            self.released_dates = None

        """
        :returns: Released Location list if available.
        """
        try:
            self.released_locations = self.releases_df.Location.tolist()
        except:
            self.released_locations = None

        """
        :returns: Released Date in India if available.
        """
        try:
            releases_index = [i for i in range(len(soup.select('h4'))) if 'release' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
            releases = soup.select('h4')[releases_index].findNext('table').select('tr')
            india_index = [i for i in range(len(releases)) if 'india' in  releases[i].select_one('td').a.text.strip().lower()][0]
            self.release_date_in_india = releases[india_index].select_one('td').findNext('td').text.strip()
        except:
            self.release_date_in_india = None

        """
        returns: Also Known As DataFrame if available.
        """
        aka_index = [i for i in range(len(soup.select('h4'))) if 'also known as' in  ' '.join(soup.select('h4')[i].text.split()).lower()][0]
        aka = soup.select('h4')[aka_index].findNext('table').select('tr')
        self.also_known_as_df = pd.DataFrame(columns=['Country', 'Title'])
        try:
            for i in range(len(aka)):
                try:
                    self.also_known_as_df.loc[len(self.also_known_as_df)] = [aka[i].select_one('td.aka-item__name').text.strip(),
                                                                             aka[i].select_one('td.aka-item__title').text.strip()]
                except:
                    pass
        except:
            self.also_known_as_df = None

        """
        :returns: Also Known As Countries list if available.
        """
        try:
            self.also_known_as_country_names = self.also_known_as_df.Country.tolist()
        except:
            self.also_known_as_country_names = None

        """
        :returns: Also Known As Title list if available.
        """
        try:
            self.also_known_as_titles = self.also_known_as_df.Title.tolist()
        except:
            self.also_known_as_titles = None

        """
        :returns: Creates Meta Data from the above info. if available.
        """
        try:
            self.imdb_release_info_metadata = {"Movie Title" : self.title,
                                               "Title ID" : self.titleid,
                                               "Movie Release Info URL" : self.release_info_url,
                                               "India Release Date" : self.release_date_in_india,
                                               "Release Dates" : {"Country" : self.released_country_names,
                                                                  "Date" : self.released_dates,
                                                                  "Location" : self.released_locations},
                                               "Also Known As (AKA)" : {"Country" : self.also_known_as_country_names,
                                                                        "Title" : self.also_known_as_titles}}
        except:
            self.imdb_release_info_metadata = None

        print("\rRelease Info Extraction Completed\r", end="\r", flush=True)



from imdby.utils import *


# Retrieves IMDb Plot Details
class plot:

    """
    Collects IMDb Plot Details of the multi-media content in IMDb when titleid is given.
    :param titleid: Unique identification for every multimedia in IMdb.
    """

    def __init__(self, titleid):
        self.titleid = titleid
        self.plot_url = "https://www.imdb.com/title/%s/plotsummary" % self.titleid
        soup = BeautifulSoup(get(self.plot_url).text, 'lxml')

        """
        :returns: Movie Title
        """
        try:
            self.title = soup.select_one('h3[itemprop="name"]').a.text.strip()
        except:
            self.title = None

        """
        :returns: Movie Plot
        """
        try:
            self.plot = ' '.join(soup.select_one('#synopsis').findNext('ul').text.split()).replace("\'", " ")
        except:
            self.plot = None

        """
        :returns: Movies Summaries
        """
        try:
            block = soup.select_one('#summaries').findNext('ul').select('li')
            self.summaries = [' '.join(block[i].text.split()) for i in range(len(block))]
        except:
            self.summaries = None

        """
        :returns: Creates Dict from the above info. if available.
        """
        try:
            self.imdb_plot_metadata = {"Movie Name" : self.title,
                                       "Title ID" : self.titleid,
                                       "Movie Plot URL" : self.plot_url,
                                       "Plot" : self.plot,
                                       "Summaries" : self.summaries}
        except:
            self.imdb_plot_metadata = None

        print("\rPlot Extraction Completed\r", end="\r", flush=True)
